@@ -14,9 +14,11 @@ from aim.engine.utils import (
     deep_compare,
     import_module,
     clean_repo_path,
+    get_dict_item_by_path,
 )
 from aim.engine.profile import AimProfile
 from aim.engine.repo.run import Run
+from aim.engine.repo.dql.select import SelectResult
 from aim.engine.repo.utils import (
     cat_to_dir,
     get_experiment_path,
@@ -245,6 +247,40 @@ class AimRepo:
         """
         return ls_dir([self.path])
 
+    def reconstruct_meta_file(self):
+        """
+        Reconstruct meta file(`Metric` and `NestedMap` artifacts)
+        from tracked artifacts data.
+        NOTE: Only can be needed in very specific cases.
+        """
+        meta_file_content = {}
+
+        # Check if `NestedMap` were saved
+        map_path = os.path.join(self.objects_dir_path, 'map', 'dictionary.log')
+        if os.path.isfile(map_path):
+            meta_file_content['dictionary.log'] = {
+                'name': 'dictionary',
+                'type': ['map', 'nested_map'],
+                'data': None,
+                'data_path': 'map',
+            }
+
+        # Collect metrics meta info
+        metrics_info = self.records_storage.get_artifacts_names()
+        for metric_name, context_items in metrics_info.items():
+            meta_file_content[metric_name] = {
+                'name': metric_name,
+                'type': 'metrics',
+                'data': None,
+                'data_path': '__AIMRECORDS__',
+                'format': {
+                    'artifact_format': 'aimrecords',
+                    'record_format': 'protobuf',
+                },
+                'context': [list(c.items()) for c in context_items],
+            }
+        return meta_file_content
+
     def load_meta_file(self, create_if_not_exist=True):
         if self.meta_file_content is None:
             if os.path.isfile(self.meta_file_path):
@@ -285,9 +321,9 @@ class AimRepo:
             if updated:
                 self.flush_meta_file()
 
-    def flush_meta_file(self):
+    def flush_meta_file(self, content=None):
         with open(self.meta_file_path, 'w+') as meta_file:
-            meta_file.write(json.dumps(self.meta_file_content))
+            meta_file.write(json.dumps(content or self.meta_file_content))
 
     def store_dir(self, name, cat, data={}):
         """
@@ -566,7 +602,7 @@ class AimRepo:
 
     def list_branch_commits(self, branch):
         """
-        Returns list of specified branches commits
+        Returns list of specified branch commits
         """
         branch_path = os.path.join(self.path, branch.strip())
 
@@ -574,9 +610,7 @@ class AimRepo:
         for i in os.listdir(branch_path):
             if os.path.isdir(os.path.join(branch_path, i)) \
                     and i != AIM_COMMIT_INDEX_DIR_NAME:
-
                 commits.append(i)
-
         return commits
 
     def is_index_empty(self):
@@ -584,9 +618,7 @@ class AimRepo:
         Returns `True` if index directory is empty and
         `False` otherwise
         """
-        if len(ls_dir([self.index_path])):
-            return False
-        return True
+        return not len(ls_dir([self.index_path]))
 
     def get_latest_vc_branch(self):
         """
@@ -840,6 +872,90 @@ class AimRepo:
         commit_path = os.path.join(self.path, branch, commit)
         return ls_dir([commit_path])
 
+    def select(self,
+               select_fields: List[str] = [],
+               expression: Optional[
+                       Union[str,
+                             Expression,
+                             BinaryExpressionTree]] = None,
+               default_expression: Optional[
+                       Union[str,
+                             Expression,
+                             BinaryExpressionTree]] = None,
+               ):
+        select_result = SelectResult(select_fields)
+
+        runs = {
+            exp_name: [
+                Run(self, exp_name, run_hash)
+                for run_hash in self.list_branch_commits(exp_name)
+            ]
+            for exp_name in self.list_branches()
+        }
+
+        # Build expression tree
+        if expression:
+            expression = build_bet(expression)
+            expression.strict = True
+
+        if default_expression:
+            default_expression = build_bet(default_expression)
+            default_expression.strict = True
+            if expression:
+                expression.concat(default_expression)
+            else:
+                expression = default_expression
+
+        for experiment_runs in runs.values():
+            for run in experiment_runs:
+                # Dictionary representing all search fields
+                fields = {
+                    'experiment': run.experiment_name,
+                    'run': run.config,  # Run configs (date, name, archived etc)
+                    'params': run.params,  # Run parameters (`NestedMap`)
+                }
+                # Default parameters - those passed without namespace
+                default_params = {
+                    'params': (run.params.get(AIM_NESTED_MAP_DEFAULT) or {}),
+                }
+
+                # Search metrics
+                for metric_name, metric in run.get_all_metrics().items():
+                    fields['metric'] = metric_name
+                    for trace in metric.get_all_traces():
+                        fields['context'] = trace.context
+                        # Pass fields in descending order by priority
+                        if expression is None:
+                            res = True
+                        else:
+                            res = expression.match(fields,
+                                                   run.params,
+                                                   default_params)
+
+                        if res is not True:
+                            continue
+
+                        # Append trace data if metric is selected
+                        for select_field in select_fields:
+                            if select_field == metric_name:
+                                metric.append(trace)
+                                run.add(metric)
+                                break
+
+                        # Append run if either metric or param is selected
+                        for select_field in select_fields:
+                            if select_field == metric_name:
+                                select_result.append_run(run)
+                                break
+
+                            field_val = get_dict_item_by_path(run.params,
+                                                              select_field)
+                            if field_val is not None:
+                                select_result.append_run(run)
+                                break
+
+        return select_result
+
     def select_runs(self,
                     expression: Optional[
                            Union[str,
@@ -860,7 +976,7 @@ class AimRepo:
 
         matched_runs = []  # type: List[Run]
 
-        # Build expression trees
+        # Build expression tree
         if expression:
             expression = build_bet(expression)
             expression.strict = True
