@@ -27,7 +27,7 @@ def deep_merge(*dicts, update=False):
         return reduce(merge_into, dicts, {})
 
 
-def deep_compare(a, b, order_matter=True):
+def deep_compare(a, b, order_matters=True):
     return json.dumps(a) == json.dumps(b)
     # TODO: Implement deep comparison not taking into account ordering
 
@@ -55,6 +55,37 @@ def ls_dir(paths):
         return [paths[0]] + (ls_dir(paths[1:]) if len(paths) > 1 else [])
 
 
+def get_dict_item_by_path(haystack, path):
+    sub_haystack = haystack
+    for token in path.split('.'):
+        if isinstance(sub_haystack, dict) and token in sub_haystack.keys():
+            sub_haystack = sub_haystack[token]
+        else:
+            break
+    else:
+        return sub_haystack
+    return None
+
+
+def contexts_equal(a, b):
+    if isinstance(a, dict):
+        a = tuple(sorted(a.items()))
+
+    if isinstance(b, dict):
+        b = tuple(sorted(b.items()))
+
+    if len(a) != len(b):
+        return False
+
+    for other_k, other_v in a:
+        for k, v in b:
+            if k == other_k and v == other_v:
+                break
+        else:
+            return False
+    return True
+
+
 def random_str(string_length=10):
     """
     Generate a random string of fixed length
@@ -62,6 +93,19 @@ def random_str(string_length=10):
     return ''.join(choice(ascii_letters) for _ in range(string_length))
 
 
+def compressed_dict(item):
+    stack = list(item.items())
+    ans = {}
+    while stack:
+        key, val = stack.pop()
+        if isinstance(val, dict):
+            for sub_key, sub_val in val.items():
+                stack.append(('{}.{}'.format(key, sub_key), sub_val))
+        else:
+            ans[key] = val
+    return ans
+
+
 def get_inst_type_str(inst):
     """
     Get instance type and class type full names
