@@ -12,6 +12,7 @@ from aim.sdk.session.utils import (
 )
 from aim.sdk.session.configs import DEFAULT_FLUSH_FREQUENCY
 from aim.artifacts import *
+from aim.engine.utils import contexts_equal
 from aim.engine.configs import (
     AIM_BRANCH_ENV_VAR,
     AIM_COMMIT_ENV_VAR,
@@ -31,7 +32,8 @@ class Session:
     @exception_resistant
     def __init__(self, repo: Optional[str] = None,
                  experiment: Optional[str] = None,
-                 flush_frequency: int = DEFAULT_FLUSH_FREQUENCY):
+                 flush_frequency: int = DEFAULT_FLUSH_FREQUENCY,
+                 block_termination: bool = True):
         self.active = False
         self._lock = threading.Lock()
         self._close_lock = threading.Lock()
@@ -57,6 +59,7 @@ class Session:
         self._queue_worker_th = threading.Thread(target=self._queue_worker)
         self._queue_worker_th.daemon = True
         self._queue_worker_th.start()
+        self._block_termination = block_termination
 
         # Bind signal listeners
         self._set_exit_handlers()
@@ -103,11 +106,13 @@ class Session:
 
         if should_lock:
             with self._lock:
-                if should_check_sess_status and not self.active:
+                if should_check_sess_status \
+                        and not self.active and not self._block_termination:
                     return
                 return self._track_body(*args, **kwargs)
         else:
-            if should_check_sess_status and not self.active:
+            if should_check_sess_status \
+                    and not self.active and not self._block_termination:
                 return
             return self._track_body(*args, **kwargs)
 
@@ -144,7 +149,7 @@ class Session:
         obj = globals()[artifact_name]
 
         # Create an instance
-        inst = obj(*args, **kwargs, __aim_session_id=id(self))
+        inst = obj(*args, **kwargs, __aim_session_id=self.run_hash)
 
         # Collect metrics values
         if isinstance(inst, Metric):
@@ -159,7 +164,7 @@ class Session:
     @exception_resistant
     def _flush(self):
         with self._lock:
-            if not self.active:
+            if not self.active and not self._block_termination:
                 return
             self._flush_metrics(force=True, check_status=True)
 
@@ -204,7 +209,8 @@ class Session:
         self._metrics_flush.setdefault(metric_inst.name, [0, 0])
         self._metrics_flush[metric_inst.name][0] = metric_inst.step
         for metric_item in self.metrics[metric_inst.name]:
-            if metric_item['context'] == metric_inst.hashable_context:
+            if contexts_equal(metric_item['context'],
+                                metric_inst.hashable_context):
                 if value < metric_item['values']['min']:
                     metric_item['values']['min'] = value
                 if value > metric_item['values']['max']:
